using System;
using System.Collections;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using Telerik.Web.UI;

namespace MyCustomNamespace
{
    public class MyTemplatedColumn : GridTemplateColumn
    {
        public MyTemplatedColumn()
        {
        }

        protected DateTime? EndSelectedDate
        {
            get
            {
                object date = ViewState["EndSelectedDate"];
                if (date == null)
                {
                    date = DateTime.MinValue;
                }

                return (DateTime) date;
            }
            set { ViewState["EndSelectedDate"] = value; }
        }

        protected DateTime? StartSelectedDate
        {
            get
            {
                object date = ViewState["StartSelectedDate"];
                if (date == null)
                {
                    date = DateTime.MinValue;
                }

                return (DateTime) date;
            }
            set { ViewState["StartSelectedDate"] = value; }
        }

        private string ComboId
        {
            get { return string.Format("RadComboBox1_{0}", this.UniqueName); }
        }

        public void ClearFiltersText()
        {
            EndSelectedDate = null;
            StartSelectedDate = null;
            this.CurrentFilterValue = "asd";
        }

        public override void Initialize()
        {
            base.Initialize();
            Owner.OwnerGrid.ItemCommand += new GridCommandEventHandler(OwnerGrid_ItemCommand);
        }

        private void OwnerGrid_ItemCommand(object source, GridCommandEventArgs e)
        {
            if (e.CommandArgument.ToString() == this.UniqueName)
            {
                if (e.CommandName == "DateFilter")
                {
                    RadComboBox comboBox =
                        ((GridFilteringItem) e.Item)[this.UniqueName].FindControl(
                            ComboId) as RadComboBox;

                    if (comboBox != null)
                    {
                        RadDatePicker radDatePicker1 =
                            comboBox.SelectedItem.FindControl("RadDatePicker1") as RadDatePicker;
                        RadDatePicker radDatePicker2 =
                            comboBox.SelectedItem.FindControl("RadDatePicker2") as RadDatePicker;

                        if (radDatePicker1 != null && radDatePicker2 != null)
                        {
                            StartSelectedDate = radDatePicker1.SelectedDate;
                            EndSelectedDate = radDatePicker2.SelectedDate;

                            // if the filter expression is not empty we might need to clear some parts of it 
                            // i.e. every filtering part that is associated with the columnName we are currently processing
                            // then we need to concatenate the new filtering parts with the remaining ones (not replace them)
                            e.Item.OwnerTableView.FilterExpression =
                                ClearFilterExpression(e.Item.OwnerTableView.FilterExpression, this.UniqueName);

                            if (e.Item.OwnerTableView.FilterExpression != "")
                            {
                                e.Item.OwnerTableView.FilterExpression +=
                                    String.Format(" AND {0} >= '{1}' AND {0} <= '{2}'",
                                                  this.DataField,
                                                  radDatePicker1.SelectedDate,
                                                  radDatePicker2.SelectedDate);
                            }
                            else
                            {
                                e.Item.OwnerTableView.FilterExpression +=
                                    String.Format("{0} >= '{1}' AND {0} <= '{2}'",
                                                  this.DataField,
                                                  radDatePicker1.SelectedDate,
                                                  radDatePicker2.SelectedDate);
                            }

                            e.Item.OwnerTableView.Rebind();
                        }
                    }
                }
                else if (e.CommandName == "Clear")
                {
                    RadComboBox comboBox =
                        ((GridFilteringItem) e.Item)[this.UniqueName].FindControl(
                            ComboId) as RadComboBox;

                    if (comboBox != null)
                    {
                        RadDatePicker radDatePicker1 =
                            comboBox.SelectedItem.FindControl("RadDatePicker1") as RadDatePicker;
                        RadDatePicker radDatePicker2 =
                            comboBox.SelectedItem.FindControl("RadDatePicker2") as RadDatePicker;

                        if (radDatePicker1 != null && radDatePicker2 != null)
                        {
                            ClearFiltersText();
                        }
                    }
                }
            }
        }

        private static string ClearFilterExpression(string filterExpression, string columnName)
        {
            string[] filterExpressionParts = filterExpression.Split(new string[] {" AND "}, StringSplitOptions.None);
            ArrayList list = new ArrayList(filterExpressionParts);
            for (int i = list.Count - 1; i >= 0; i--)
            {
                if (list[i].ToString().StartsWith(columnName))
                {
                    list.Remove(list[i]);
                }
            }

            filterExpressionParts = (string[]) list.ToArray(typeof (string));

            return string.Join(" AND ", filterExpressionParts);
        }

        protected override void SetupFilterControls(TableCell cell)
        {
            RadComboBox combo = new RadComboBox();
            combo.ID = ComboId;
            combo.Skin = Owner.OwnerGrid.Skin;
            combo.DropDownWidth = Unit.Pixel(320);
            combo.CloseDropDownOnBlur = false;
            combo.ItemTemplate = new ComboTemplate();

            RadComboBoxItem item = new RadComboBoxItem();
            item.Selected = true;
            combo.Items.Add(item);

            Button buttonOK = combo.SelectedItem.FindControl("ButtonOK") as Button;
            if (buttonOK != null)
            {
                buttonOK.CommandArgument = this.UniqueName;
            }

            Button buttonClear = combo.SelectedItem.FindControl("ButtonClear") as Button;
            if (buttonClear != null)
            {
                buttonClear.CommandArgument = this.UniqueName;
            }

            cell.Controls.Add(combo);

            cell.Controls.Add(
                new LiteralControl(
                    String.Format("<script type='text/javascript'>PrepareComboForDisposal('{0}');</script>",
                                  combo.ClientID)));
        }

        protected override string GetCurrentFilterValueFromControl(TableCell cell)
        {
            RadComboBox combo = (RadComboBox) cell.Controls[0];
            RadDatePicker dpStart = (RadDatePicker) combo.SelectedItem.FindControl("RadDatePicker1");
            RadDatePicker dpEnd = (RadDatePicker) combo.SelectedItem.FindControl("RadDatePicker2");

            string currentValue = "";

            if (!dpStart.IsEmpty && !dpEnd.IsEmpty)
            {
                this.CurrentFilterFunction = GridKnownFunction.Between;
                currentValue = string.Format("{0} AND {1}", dpStart.SelectedDate, dpEnd.SelectedDate);
            }
            else
            {
                this.CurrentFilterFunction = GridKnownFunction.NoFilter;
            }
            return currentValue;
        }

        protected override void SetCurrentFilterValueToControl(TableCell cell)
        {
            base.SetCurrentFilterValueToControl(cell);
            try
            {
                RadComboBox comboBox = cell.FindControl(ComboId) as RadComboBox;

                if (comboBox != null)
                {
                    RadDatePicker dpStartDate = comboBox.SelectedItem.FindControl("RadDatePicker1") as RadDatePicker;
                    RadDatePicker dpEndDate = comboBox.SelectedItem.FindControl("RadDatePicker2") as RadDatePicker;

                    if (dpStartDate != null && StartSelectedDate != DateTime.MinValue)
                    {
                        dpStartDate.SelectedDate = (DateTime)StartSelectedDate;
                        dpStartDate.FocusedDate = (DateTime)StartSelectedDate;
                    }

                    if (dpEndDate != null && EndSelectedDate != DateTime.MinValue)
                    {
                        dpEndDate.SelectedDate = (DateTime)EndSelectedDate;
                        dpEndDate.FocusedDate = (DateTime)EndSelectedDate;
                    }
                }
            }
            catch (Exception e)
            {
            }
        }

        #region Nested type: ComboTemplate

        private class ComboTemplate : ITemplate
        {
            #region ITemplate Members

            public void InstantiateIn(Control container)
            {
                HtmlTable table = new HtmlTable();

                table.Style["width"] = Unit.Percentage(100).ToString();
                table.Style["height"] = Unit.Percentage(100).ToString();

                HtmlTableRow row = new HtmlTableRow();
                RadDatePicker startPicker = new RadDatePicker();
                startPicker.ID = "RadDatePicker1";
                startPicker.FocusedDate = new DateTime(1996, 7, 1);
                HtmlTableCell cell = new HtmlTableCell();
                cell.Controls.Add(startPicker);
                row.Controls.Add(cell);

                RadDatePicker endPicker = new RadDatePicker();
                endPicker.ID = "RadDatePicker2";
                endPicker.FocusedDate = new DateTime(1996, 7, 1);
                cell = new HtmlTableCell();
                cell.Controls.Add(endPicker);
                row.Controls.Add(cell);

                row.Attributes.Add("onclick", "PreventDropDownClose(event);");
                table.Rows.Add(row);

                row = new HtmlTableRow();
                cell = new HtmlTableCell();
                Button btn = new Button();
                btn.ID = "ButtonOK";
                btn.Text = "OK";
                btn.CommandName = "DateFilter";
                btn.Width = Unit.Pixel(40);
                cell.Controls.Add(btn);
                row.Controls.Add(cell);

                cell = new HtmlTableCell();
                btn = new Button();
                btn.ID = "ButtonClear";
                btn.Text = "Clear";
                btn.CommandName = "Clear";
                btn.Width = Unit.Pixel(40);
                cell.Controls.Add(btn);
                row.Controls.Add(cell);
                table.Rows.Add(row);

                container.Controls.Add(table);
            }

            #endregion
        }

        #endregion
    }
}